precision mediump float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float inputHeight;
uniform float inputWidth;
uniform float textHeight;
uniform float textWidth;
uniform highp float time;
uniform highp float duration;

vec2 getBezierValue1(float controls[4], float t) {
    float xc1 = controls[0];
    float yc1 = controls[1];
    float xc2 = controls[2];
    float yc2 = controls[3];
    float a = 3*xc1*(1-t)*(1-t)*t+3*xc2*(1-t)*t*t+t*t*t;
    float b = 3*yc1*(1-t)*(1-t)*t+3*yc2*(1-t)*t*t+t*t*t;
    return vec2(a, b);
}

float getBezierTfromX1(float controls[4], float x) {
    float ts = 0.;
    float te = 1.;
    while (te-ts >= 0.001) {
        float tm = (ts+te)/2.;
        vec2 value = getBezierValue1(controls, tm);
        if (value.x>x) {
            te = tm;
        } else {
            ts = tm;
        }
    }
    return (te+ts)/2;
}

float toBezier(float t, float P0, float P1, float P2, float P3) {
    float controls[4];
    controls[0] = P0;
    controls[1] = P1;
    controls[2] = P2;
    controls[3] = P3;
    float tvalue = getBezierTfromX1(controls, t);
    vec2 value = getBezierValue1(controls, tvalue);
    return value.y;
}

vec4 directionBlur(sampler2D tex, vec2 resolution, vec2 uv, vec2 directionOfBlur, float intensity) {
    float blurStep = 3.;
    vec2 pixelStep = 1.0/resolution * blurStep * intensity;
    float dircLength = length(directionOfBlur);
    if (dircLength > 0.01)
    {
        pixelStep.x = directionOfBlur.x * 1.0 / dircLength * pixelStep.x;
        pixelStep.y = directionOfBlur.y * 1.0 / dircLength * pixelStep.y;
    }

    vec4 color = vec4(0.);
    int num = 10;
    for (int i = -num; i <= num; i++)
    {
        vec2 blurCoord = uv + pixelStep * float(i);
        blurCoord = abs(blurCoord);
        if (blurCoord.x > 1.0)
        blurCoord.x = 1.0 - fract(blurCoord.x);
        if (blurCoord.y > 1.0)
        blurCoord.y = 1.0 - fract(blurCoord.y);

        color += texture2D(tex, blurCoord);
    }
    color /= float(2 * num + 1);
    return color;
}

void main() {
    float width = inputWidth;
    float height = inputHeight;
    vec2 resolution = vec2(width, height);
    vec2 uv0 = textureCoordinate;
    float timeValue = time;
    float durationValue = duration;
    float progress = timeValue/durationValue;

    progress = toBezier(progress, 0.28, 0.7, 0.48, 0.82);
    vec2 direction = vec2(0, resolution.y);
    vec4 resultColor = directionBlur(inputImageTexture, resolution, uv0, direction, (1.0-progress) * 2.0);
    gl_FragColor = resultColor;
}
